#include <SPI.h>
#include <MFRC522.h> //MFRC522 library
#include <LiquidCrystal_I2C.h> // LiquidCrystal I2C - Version: Latest 

#define SS_PIN 10
#define RST_PIN 9
#define LED_G 5 //define green LED pin
#define LED_R 4 //define red LED
#define RELAY 8 //relay pin
#define BUZZER 3 //buzzer pin
#define ACCESS_DELAY 2000
#define DENIED_DELAY 1000

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance.
LiquidCrystal_I2C lcd(0x27,20,4);   // create lcd instance

String card_good; // string to memorize good uid

void setup() 
{
  Serial.begin(9600);   // Initiate a serial communication
  SPI.begin();          // Initiate  SPI bus
  mfrc522.PCD_Init();   // Initiate MFRC522
  pinMode(LED_G, OUTPUT);
  pinMode(LED_R, OUTPUT);
  pinMode(RELAY, OUTPUT);
  pinMode(BUZZER, OUTPUT);
  
  noTone(BUZZER);
  digitalWrite(RELAY, HIGH);
  
  Serial.print("Show good card");
  Serial.println();
  
  lcd.init();
  lcd.init();
  lcd.backlight();
  lcd.setCursor(1,0);
  lcd.print("Show good card");  // Print a message to the LCD.
  
  card_good = "";
}
void loop() 
{
  // if already memorized uid, wait for card to be placed
  if(card_good != "") {
    lcd.clear();
    lcd.backlight();
    lcd.setCursor(3,0);
    lcd.print("Place card");
  }
  // Look for new cards
  if ( ! mfrc522.PICC_IsNewCardPresent()) 
  {
    return;
  }
  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) 
  {
    return;
  }
  
  // if uid not yet memorized, wait for good card to be placed
  if(card_good == "") {
    byte letter;
    Serial.print("UID authorized tag :");
    for (byte i = 0; i < mfrc522.uid.size; i++) 
    {
       Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
       Serial.print(mfrc522.uid.uidByte[i], HEX);
       card_good.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
       card_good.concat(String(mfrc522.uid.uidByte[i], HEX));
    }
    card_good.toUpperCase();
    Serial.println();
    Serial.print("Memorized");
    Serial.println();
    
    lcd.setCursor(3,1);
    lcd.print("Memorized");
    delay(2000);
    return;
  }
  
  //Show UID on serial monitor
  Serial.print("UID tag :");
  String content= "";
  byte letter;
  for (byte i = 0; i < mfrc522.uid.size; i++) 
  {
     Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
     Serial.print(mfrc522.uid.uidByte[i], HEX);
     content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
     content.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
  Serial.println();
  Serial.print("Message : ");
  content.toUpperCase();
  
  //compare current uid with good uid
  if (content.substring(1).equals(card_good.substring(1)))
  {
    //if match, authorize access
    Serial.println("Authorized access");
    Serial.println();
    delay(500);
    
    digitalWrite(RELAY, LOW);
    digitalWrite(LED_G, HIGH);
    
    lcd.clear();
    lcd.setCursor(3,0);
    lcd.print("Card placed");
    lcd.setCursor(0,1);
    lcd.print("Authorized access");
    
    delay(ACCESS_DELAY);
    digitalWrite(RELAY, HIGH);
    digitalWrite(LED_G, LOW);
  }
 
 else   {
    // if not, deny access
    Serial.println(" Access denied");
    digitalWrite(LED_R, HIGH);
    tone(BUZZER, 300);
    
    lcd.clear();
    lcd.setCursor(3,0);
    lcd.print("Card placed");
    lcd.setCursor(1,1);
    lcd.print("Access denied!");
    
    delay(DENIED_DELAY);
    digitalWrite(LED_R, LOW);
    noTone(BUZZER);
  }
}